@@ -1,4 +1,4 @@
-/*  Copyright (C) 2023 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2024 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -20,6 +20,7 @@
 #include <stdlib.h>
 #include <urcu.h>
 
+#include "knot/nameserver/process_query.h"
 #include "knot/server/handler.h"
 #include "knot/server/quic-handler.h"
 #include "knot/server/xdp-handler.h"
@@ -196,6 +197,12 @@ static void handle_udp(xdp_handle_ctx_t *ctx, knot_layer_t *layer,
 			continue;
 		}
 
+		params_xdp_update(params, KNOTD_QUERY_PROTO_UDP, msg_recv);
+
+		if (process_query_proto(params, KNOTD_STAGE_PROTO_BEGIN) == KNOTD_PROTO_STATE_BLOCK) {
+			continue;
+		}
+
 		// Try to allocate a buffer for a reply.
 		if (knot_xdp_reply_alloc(ctx->sock, msg_recv, msg_send) != KNOT_EOK) {
 			if (log_enabled_debug()) {
@@ -206,36 +213,50 @@ static void handle_udp(xdp_handle_ctx_t *ctx, knot_layer_t *layer,
 		ctx->msg_udp_count++;
 
 		// Prepare a reply.
-		params_xdp_update(params, KNOTD_QUERY_PROTO_UDP, msg_recv, 0, NULL);
 		handle_udp_reply(params, layer, &msg_recv->payload, &msg_send->payload,
 		                 &proxied_remote);
+
+		(void)process_query_proto(params, KNOTD_STAGE_PROTO_END);
 	}
 }
 
 static void handle_tcp(xdp_handle_ctx_t *ctx, knot_layer_t *layer,
                        knotd_qdata_params_t *params)
 {
-	int ret = knot_tcp_recv(ctx->relays, ctx->msg_recv, ctx->msg_recv_count,
-	                        ctx->tcp_table, ctx->syn_table, XDP_TCP_IGNORE_NONE);
-	if (ret != KNOT_EOK) {
-		if (log_enabled_debug()) {
-			log_debug("TCP/XDP, failed to process some packets (%s)", knot_strerror(ret));
-		}
-		return;
-	} else if (knot_tcp_relay_empty(&ctx->relays[0])) { // no TCP traffic
-		return;
-	}
 
 	uint8_t ans_buf[KNOT_WIRE_MAX_PKTSIZE];
 
 	for (uint32_t i = 0; i < ctx->msg_recv_count; i++) {
+		knot_xdp_msg_t *msg_recv = &ctx->msg_recv[i];
 		knot_tcp_relay_t *rl = &ctx->relays[i];
 
+		if (!(msg_recv->flags & KNOT_XDP_MSG_TCP)) {
+			continue;
+		}
+
+		params_xdp_update(params, KNOTD_QUERY_PROTO_TCP, msg_recv);
+
+		if (process_query_proto(params, KNOTD_STAGE_PROTO_BEGIN) == KNOTD_PROTO_STATE_BLOCK) {
+			continue;
+		}
+
+		int ret = knot_tcp_recv(rl, msg_recv, ctx->tcp_table,
+		                        ctx->syn_table, XDP_TCP_IGNORE_NONE);
+		if (ret != KNOT_EOK) {
+			if (log_enabled_debug()) {
+				log_debug("TCP/XDP, failed to process some packets (%s)",
+				          knot_strerror(ret));
+			}
+			continue;
+		} else if (knot_tcp_relay_empty(rl)) {
+			continue;
+		}
+
+		params_update_tcp(params, rl->conn->establish_rtt);
+
 		// Process all complete DNS queries in one TCP stream.
 		for (size_t j = 0; rl->inbf != NULL && j < rl->inbf->n_inbufs; j++) {
 			// Consume the query.
-			params_xdp_update(params, KNOTD_QUERY_PROTO_TCP, ctx->msg_recv,
-			                  rl->conn->establish_rtt, NULL);
 			struct iovec *inbufs = rl->inbf->inbufs;
 			handle_query(params, layer, &inbufs[j], NULL);
 
@@ -253,6 +274,8 @@ static void handle_tcp(xdp_handle_ctx_t *ctx, knot_layer_t *layer,
 
 			handle_finish(layer);
 		}
+
+		(void)process_query_proto(params, KNOTD_STAGE_PROTO_END);
 	}
 }
 
@@ -274,6 +297,12 @@ static void handle_quic(xdp_handle_ctx_t *ctx, knot_layer_t *layer,
 			continue;
 		}
 
+		params_xdp_update(params, KNOTD_QUERY_PROTO_QUIC, msg_recv);
+
+		if (process_query_proto(params, KNOTD_STAGE_PROTO_BEGIN) == KNOTD_PROTO_STATE_BLOCK) {
+			continue;
+		}
+
 		knot_quic_reply_t *reply = &ctx->quic_replies[i];
 		knot_xdp_msg_t *msg_out = &ctx->msg_send_udp[i];
 
@@ -289,7 +318,9 @@ static void handle_quic(xdp_handle_ctx_t *ctx, knot_layer_t *layer,
 		                       &ctx->quic_relays[i]);
 		knot_quic_conn_t *conn = ctx->quic_relays[i];
 
-		handle_quic_streams(conn, params, layer, &ctx->msg_recv[i]);
+		handle_quic_streams(conn, params, layer);
+
+		(void)process_query_proto(params, KNOTD_STAGE_PROTO_END);
 	}
 #else
 	(void)(ctx);
